package jalview.gui;

import java.awt.*;
import java.awt.event.*;
import jalview.datamodel.*;
import javax.swing.*;
import jalview.schemes.*;
import jalview.analysis.*;


public class SeqPanel extends JPanel
{

  public    SeqCanvas         seqCanvas;
  public    AlignmentPanel    parent;

  protected int startres;
  protected int lastres;
  protected int endres;

  protected int startseq;
  protected int padseq;

  protected AlignViewport av;

  // if character is inserted or deleted, we will need to recalculate the conservation
  int seqEditOccurred = -1;

  public SeqPanel(AlignViewport avp, AlignmentPanel p) {
    this.av         = avp;

    seqCanvas  = new SeqCanvas(avp);
    setLayout(new BorderLayout());
    add(seqCanvas, BorderLayout.CENTER);

    parent = p;

    addMouseMotionListener( new MouseMotionAdapter()
    {
      public void mouseMoved(MouseEvent evt)
      {   doMouseMoved(evt);    }

      public void mouseDragged(MouseEvent evt)
      {
        if(evt.isShiftDown() || evt.isAltDown() || evt.isControlDown())
          doMouseDragged(evt);
        else
          doMouseDraggedDefineMode(evt);
      }
    });

    addMouseListener( new MouseAdapter()
    {
      public void mouseReleased(MouseEvent evt)
      {
        if(evt.isShiftDown() || evt.isAltDown() || evt.isControlDown())
          doMouseReleased(evt);
        else
          doMouseReleasedDefineMode(evt);
      }
      public void mousePressed(MouseEvent evt)
      {
        if(evt.isShiftDown() || evt.isAltDown() || evt.isControlDown())
          doMousePressed(evt);
        else
          doMousePressedDefineMode(evt);
      }

    });
    repaint();
  }


  public void doMouseReleased(MouseEvent evt) {

    int x = evt.getX();
    int res = x/av.getCharWidth() + av.getStartRes();

    endres = res;

    startseq = -1;
    startres = -1;
    lastres  = -1;
    if(seqEditOccurred>-1)
      updateConservation(seqEditOccurred);

    seqEditOccurred = -1;

    parent.RefreshPanels();
    repaint();

  }

  public void doMousePressed(MouseEvent evt) {
    parent.alignFrame.addHistoryItem("sequence edit");
    int seq;
    int res;

    int x = evt.getX();
    int y = evt.getY();

    res = x/av.getCharWidth() + av.getStartRes();
    seq = y/av.getCharHeight() + av.getStartSeq();

    if (seq < av.getAlignment().getHeight() &&
        res < av.getAlignment().getSequenceAt(seq).getLength())
    {
      //char resstr = align.getSequenceAt(seq).getSequence().charAt(res);
      // Find the residue's position in the sequence (res is the position
      // in the alignment

      startseq = seq;

      if (startseq == (av.getAlignment().getHeight() - 1))
        padseq = 1;
      else
        padseq = 1;

      startres = res;
      lastres = res;

    }
    else
    {
      startseq = -1;
      startres = -1;
      lastres = -1;
    }

    return;
  }

  public void doMouseMoved(MouseEvent evt)
  {
    int res=0, seq=0;
    int x = evt.getX();
    int y = evt.getY();
    if(av.wrapAlignment)
    {
      y -= 2*av.charHeight;
      int chunkHeight = (av.getAlignment().getHeight()+2)*av.charHeight;


      res =   (int)((y/chunkHeight)*(getWidth()/av.charWidth)) +  x/av.getCharWidth() + av.getStartRes();

      y %= chunkHeight;
      seq =     y / av.getCharHeight() + av.getStartSeq();

      //   chunkHeight =  (da.getHeight() + 2)*charHeight;
      //  startx += chunkWidth;
    }
    else
    {
      res = x / av.getCharWidth()  + av.getStartRes();
      seq = y / av.getCharHeight() + av.getStartSeq();
    }


    if(seq>=av.getAlignment().getHeight())
      return;

    Object obj = ResidueProperties.aa2Triplet.get( av.getAlignment().getSequenceAt(seq).getCharAt(res)+"" ) ;
    String aa = "";
    if(obj!=null)
         aa = obj.toString();

    StringBuffer text = new StringBuffer("Sequence " +(seq+1)+" ID: "+av.getAlignment().getSequenceAt(seq).getName());
    if(aa!="")
      text.append("  Residue: "+aa+" ("+  av.getAlignment().getSequenceAt(seq).findPosition(res)+")");

    parent.alignFrame.statusBar.setText(text.toString());

  }

  public void doMouseDragged(MouseEvent evt) {
    // If we're dragging we're editing

    int x = evt.getX();

    int res = x/av.getCharWidth() + av.getStartRes();
    if (res < 0)
      res = 0;

    if (res != lastres)
    {
	// Group editing
	if (evt.isAltDown() || evt.isControlDown())
        {
	  SequenceGroup sg = av.getAlignment().findGroup(startseq);
          if (sg != null)
          {
            boolean deleteAllowed = false;
            if (res < av.getAlignment().getWidth() && res < lastres)
            {
              /// Are we able to delete?
              boolean allGaps = true;
              for (int i = 0; i < sg.getSize(); i++)
              {
                SequenceI s = sg.getSequenceAt(i);
                for (int j = lastres-1; j >= res && allGaps; j--)
                {
                    if (!jalview.util.Comparison.isGap(s.getSequence().charAt(j)))
                    {
                      res = j + 1;
                      allGaps = false;
                    }
                }

                if(!deleteAllowed && allGaps)
                  deleteAllowed = true;
              }
            }

            // drag to right
            if (res < av.getAlignment().getWidth() && res > lastres)
                sg.setEndRes(sg.getEndRes() + 1);

            // drag to left
            else if (deleteAllowed && res < av.getAlignment().getWidth() &&
                     res < lastres)
                sg.setEndRes(sg.getEndRes() - 1);




            for (int i = 0; i < sg.getSize(); i++)
            {
              SequenceI s = sg.getSequenceAt(i);
              int k = av.alignment.findIndex(s);

              // drag to right
              if (res < av.getAlignment().getWidth() && res > lastres)
                for (int j = lastres; j < res; j++)
                  insertChar(j, k);

              // drag to left
              else if (deleteAllowed && res < av.getAlignment().getWidth() && res < lastres)
              {
                for (int j = res; j < lastres; j++)
                {
                  deleteChar(j, k);
                  startres = res;
                }
              }
            }
          }
        }
        else /////Editing a single sequence///////////
        {
          if (res < av.getAlignment().getWidth() && res > lastres)
          {
            // dragging to the right
            for (int j = lastres; j < res; j++)
              insertChar(j, startseq);
          }
          else if (res < av.getAlignment().getWidth() && res < lastres)
          {
            // dragging to the left
            for (int j = res; j < lastres; j++)
            {
              deleteChar(j, startseq);
              startres = res;
            }
	  }

	}

    }

    lastres = res;
    repaint();
  }

  public void drawChars(int seqstart, int seqend, int start) {
    seqCanvas.drawPanel(seqCanvas.gg, start,av.getEndRes(),seqstart,seqend,av.getStartRes(),av.getStartSeq(),0);
    repaint();
  }

  public void insertChar(int j, int seq)
  {
    av.alignment.getSequenceAt(seq).insertCharAt(j, av.getGapCharacter());
    seqEditOccurred=seq;
  }

  public void deleteChar(int j, int seq)
  {

    if ( jalview.util.Comparison.isGap( av.alignment.getSequenceAt(seq).getSequence().charAt(j)))
        av.alignment.getSequenceAt(seq).deleteCharAt(j);

    av.alignment.getWidth();
    repaint();
    seqEditOccurred=seq;
  }


  void updateConservation(int i)
  {
  /*  Alignment al = (Alignment) av.getAlignment();
    SequenceGroup sg = av.alignment.findGroup( al.getSequenceAt(i));
    if(sg==null || !(sg.cs instanceof ConservationColourScheme))
      return;

    Conservation c = sg.getConservation();

    c = new Conservation("All", al.cons,
                         ResidueProperties.propHash, 3, sg.sequences, 0,
                         al.getWidth());
    c.calculate();
    c.verdict(false, 100);
    sg.setConservation(c);
    ConservationColourScheme ccs = (ConservationColourScheme)sg.cs;
    ccs.conserve = c;*/
  }

//////////////////////////////////////////
/////Everything below this is for defining the boundary of the rubberband
//////////////////////////////////////////
  int oldSeq = -1;
  public void doMousePressedDefineMode(MouseEvent evt)
  {
    int res = evt.getX()/av.getCharWidth() + av.getStartRes();
    int seq = evt.getY()/av.getCharHeight() + av.getStartSeq();
    oldSeq = seq;

    stretchGroup = av.getRubberbandGroup();

    if(stretchGroup == null)
        stretchGroup = av.alignment.findGroup((Sequence)av.getAlignment().getSequenceAt(seq));

    else if(!stretchGroup.sequences.contains((Sequence)av.getAlignment().getSequenceAt(seq))
            || stretchGroup.getStartRes()>res
            || stretchGroup.getEndRes()<res)
     {
       stretchGroup = null;

       SequenceGroup[] allGroups = av.alignment.findAllGroups( (Sequence) av.
           getAlignment().
           getSequenceAt(seq));

       if (allGroups != null)
         for (int i = 0; i < allGroups.length; i++)
           if (allGroups[i].getStartRes() <= res &&
               allGroups[i].getEndRes() >= res)
           {
             stretchGroup = allGroups[i];
             av.setRubberbandGroup(stretchGroup);
             break;
           }
     }

    if(stretchGroup==null)
    {
      // define a new group here
      SequenceGroup sg = new SequenceGroup();
      sg.setStartRes(res);
      sg.setEndRes(res);
      sg.addSequence( (Sequence)av.getAlignment().getSequenceAt(seq) );
      av.setRubberbandGroup( sg );
      stretchGroup = sg;
    }
    else if( javax.swing.SwingUtilities.isRightMouseButton(evt))
    {
        jalview.gui.PopupMenu pop = new jalview.gui.PopupMenu( parent , null);
        pop.show(this, evt.getX(), evt.getY());

    // edit the properties of existing group
    }

    if(stretchGroup!=null && stretchGroup.getEndRes()==res)
      // Edit end res position of selected group
      changeEndRes = true;

   else if(stretchGroup!=null && stretchGroup.getStartRes()==res)
      // Edit end res position of selected group
      changeStartRes = true;


    seqCanvas.paintFlag = true;
    repaint();


  }

  boolean changeEndSeq = false;
  boolean changeStartSeq = false;
  boolean changeEndRes = false;
  boolean changeStartRes = false;
  SequenceGroup stretchGroup = null;

  public void doMouseReleasedDefineMode(MouseEvent evt)
  {
    changeEndRes = false;
    changeStartRes = false;
    stretchGroup = null;
  }


  boolean remove = false;
  public void doMouseDraggedDefineMode(MouseEvent evt)
  {
    int res = evt.getX()/av.getCharWidth() + av.getStartRes();
    int y = evt.getY()/av.getCharHeight() + av.getStartSeq();


    if(stretchGroup.getEndRes()==res)
      // Edit end res position of selected group
      changeEndRes = true;

    else if(stretchGroup.getStartRes()==res)
      // Edit end res position of selected group
      changeStartRes = true;


    if(res<av.getStartRes())
      res = av.getStartRes();
    else if(res>av.getEndRes())
      res = av.getEndRes();

    if(changeEndRes)
    {
      if(res>stretchGroup.getStartRes()-1)
        stretchGroup.setEndRes( res );
    }
    else if(changeStartRes)
    {
      if(res<stretchGroup.getEndRes()+1)
        stretchGroup.setStartRes( res );
    }


    int dragDirection = 0;
    if (y > oldSeq)
      dragDirection = 1;
    else if (y < oldSeq)
      dragDirection = -1;

    while (y != oldSeq)
    {
      // This routine ensures we don't skip any sequences, as the
      // selection is quite slow.
      Sequence seq = (Sequence) av.getAlignment().getSequenceAt(oldSeq);

      oldSeq += dragDirection;
      Sequence nextSeq = (Sequence) av.getAlignment().getSequenceAt(oldSeq);

      if (stretchGroup.sequences.contains(nextSeq))
      {
        stretchGroup.deleteSequence(seq);
        stretchGroup.deleteSequence(nextSeq);
      }
      else
      {
        stretchGroup.addSequence(seq);
        stretchGroup.addSequence(nextSeq);
      }
    }
    oldSeq = y;

    seqCanvas.paintFlag = true;
    repaint();
  }


}




